"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPackagesInTopologicalOrder = getPackagesInTopologicalOrder;
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const glob_1 = require("glob");
const toposort_1 = __importDefault(require("toposort"));
async function getPackagesInTopologicalOrder(monorepoRoot) {
    const patterns = JSON.parse(await fs_1.promises.readFile(path_1.default.join(monorepoRoot, `package.json`), 'utf8')).workspaces || [];
    const packageJsonPaths = await (0, glob_1.glob)(patterns.map((pattern) => `${pattern}/package.json`), { cwd: monorepoRoot });
    const internalPackages = await Promise.all(packageJsonPaths.map(async (packageJsonPath) => {
        const packageJsonLocation = path_1.default.resolve(monorepoRoot, packageJsonPath);
        const packageJson = JSON.parse(await fs_1.promises.readFile(packageJsonLocation, 'utf8'));
        return {
            name: packageJson.name,
            location: path_1.default.resolve(path_1.default.dirname(packageJsonLocation)),
            packageJson,
        };
    }));
    const edges = [];
    const packageNames = new Set(internalPackages.map((p) => p.name));
    for (const internalPackage of internalPackages) {
        const anyInternalDependency = Object.keys({
            ...(internalPackage.packageJson?.dependencies || {}),
            ...(internalPackage.packageJson?.devDependencies || {}),
            ...(internalPackage.packageJson?.optionalDependencies || {}),
            ...(internalPackage.packageJson?.peerDependencies || {}),
        }).filter((dep) => packageNames.has(dep));
        edges.push(['.root', internalPackage.name]);
        if (anyInternalDependency.length) {
            for (const dep of anyInternalDependency) {
                edges.push([internalPackage.name, dep]);
            }
        }
    }
    const sorted = (0, toposort_1.default)(edges).slice(1).reverse();
    const result = sorted
        .map((packageName) => internalPackages.find((p) => p.name === packageName))
        .map((packageInfo) => {
        if (!packageInfo)
            throw new Error('unreachable');
        return {
            name: packageInfo.name,
            version: packageInfo.packageJson.version,
            private: !!packageInfo.packageJson.private,
            location: packageInfo.location,
        };
    });
    return result;
}
//# sourceMappingURL=get-packages-in-topological-order.js.map